@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2007 Vreixo Formoso
- * Copyright (c) 2009 - 2012 Thomas Schmitt
+ * Copyright (c) 2009 - 2016 Thomas Schmitt
  *
  * This file is part of the libisofs project; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License version 2 
@@ -29,52 +29,57 @@
 #include <string.h>
 #include <stdio.h>
 
-static
-int get_iso_name(Ecma119Image *img, IsoNode *iso, char **name)
+/* @param flag   bit0=  Do not issue error messages
+*/
+int iso_get_ecma119_name(IsoWriteOpts *opts, char *input_charset, int imgid,
+                         char *node_name, enum IsoNodeType node_type,
+                         char **name, int flag)
 {
-    int ret, relaxed, free_ascii_name= 0, force_dots = 0;
+    int ret, relaxed, free_ascii_name = 0, force_dots = 0;
     char *ascii_name;
-    char *isoname= NULL;
+    char *isoname = NULL;
 
-    if (iso->name == NULL) {
+    if (node_name == NULL) {
         /* it is not necessarily an error, it can be the root */
         return ISO_SUCCESS;
     }
 
-    if (img->untranslated_name_len > 0) {
-        ascii_name = iso->name;
+    if (opts->untranslated_name_len > 0) {
+        ascii_name = node_name;
         ret = 1;
     } else {
-        ret = str2ascii(img->input_charset, iso->name, &ascii_name);
+        ret = str2ascii(input_charset, node_name, &ascii_name);
         free_ascii_name = 1;
     }
     if (ret < 0) {
-        iso_msg_submit(img->image->id, ret, 0,
-                       "Cannot convert name '%s' to ASCII", iso->name);
+        if (!(flag & 512))
+            iso_msg_submit(imgid, ret, 0,
+                           "Cannot convert name '%s' to ASCII", node_name);
         return ret;
     }
 
-    if (img->allow_full_ascii) {
+    if (opts->allow_full_ascii) {
         relaxed = 2;
     } else {
-        relaxed = (int)img->allow_lowercase;
+        relaxed = (int)opts->allow_lowercase;
     }
-    if (img->allow_7bit_ascii)
+    if (opts->allow_7bit_ascii)
         relaxed |= 4;
-    if (iso->type == LIBISO_DIR && !(img->allow_dir_id_ext)) {
-        if (img->untranslated_name_len > 0) {
-            if (strlen(ascii_name) > img->untranslated_name_len) {
+    if (node_type == LIBISO_DIR && !(opts->allow_dir_id_ext)) {
+        if (opts->untranslated_name_len > 0) {
+            if (strlen(ascii_name) > opts->untranslated_name_len) {
 needs_transl:;
-                iso_msg_submit(img->image->id, ISO_NAME_NEEDS_TRANSL, 0,
+                if (!(flag & 512))
+                    iso_msg_submit(imgid, ISO_NAME_NEEDS_TRANSL, 0,
               "File name too long (%d > %d) for untranslated recording:  '%s'",
-                               strlen(ascii_name), img->untranslated_name_len,
-                               ascii_name);
+                          strlen(ascii_name), opts->untranslated_name_len,
+                          ascii_name);
                 return ISO_NAME_NEEDS_TRANSL;
             }
             isoname = strdup(ascii_name);
-        } else if (img->max_37_char_filenames) {
+        } else if (opts->max_37_char_filenames) {
             isoname = iso_r_dirid(ascii_name, 37, relaxed);
-        } else if (img->iso_level == 1) {
+        } else if (opts->iso_level == 1) {
 
 #ifdef Libisofs_old_ecma119_nameS
 
@@ -99,14 +104,15 @@ needs_transl:;
             }
         }
     } else {
-        force_dots = !((img->no_force_dots & 1) || iso->type == LIBISO_DIR);
-        if (img->untranslated_name_len > 0) {
-            if (strlen(ascii_name) > img->untranslated_name_len)
+        force_dots = !((opts->no_force_dots & 1) ||
+                       node_type == LIBISO_DIR);
+        if (opts->untranslated_name_len > 0) {
+            if (strlen(ascii_name) > opts->untranslated_name_len)
                 goto needs_transl;
             isoname = strdup(ascii_name);
-        } else if (img->max_37_char_filenames) {
+        } else if (opts->max_37_char_filenames) {
             isoname = iso_r_fileid(ascii_name, 36, relaxed, force_dots);
-        } else if (img->iso_level == 1) {
+        } else if (opts->iso_level == 1) {
 
 #ifdef Libisofs_old_ecma119_nameS
 
@@ -151,11 +157,21 @@ needs_transl:;
     }
 }
 
+static
+int get_iso_name(Ecma119Image *img, IsoNode *iso, char **name)
+{
+    int ret;
+
+    ret = iso_get_ecma119_name(img->opts, img->input_charset, img->image->id,
+                               iso->name, iso->type, name, 0);
+    return ret;
+}
+
 int ecma119_is_dedicated_reloc_dir(Ecma119Image *img, Ecma119Node *node)
 {
     if (img->rr_reloc_node == node &&
         node != img->root && node != img->partition_root &&
-        (img->rr_reloc_flags & 2))
+        (img->opts->rr_reloc_flags & 2))
         return 1;
     return 0;
 }
@@ -185,23 +201,26 @@ static
 int create_dir(Ecma119Image *img, IsoDir *iso, Ecma119Node **node)
 {
     int ret;
-    Ecma119Node **children;
+    Ecma119Node **children = NULL;
     struct ecma119_dir_info *dir_info;
 
-    children = calloc(1, sizeof(void*) * iso->nchildren);
-    if (children == NULL) {
-        return ISO_OUT_OF_MEM;
+    if (iso->nchildren > 0) {
+        children = calloc(1, sizeof(void*) * iso->nchildren);
+        if (children == NULL)
+            return ISO_OUT_OF_MEM;
     }
 
     dir_info = calloc(1, sizeof(struct ecma119_dir_info));
     if (dir_info == NULL) {
-        free(children);
+        if (children != NULL)
+            free(children);
         return ISO_OUT_OF_MEM;
     }
 
     ret = create_ecma119_node(img, (IsoNode*)iso, node);
     if (ret < 0) {
-        free(children);
+        if (children != NULL)
+            free(children);
         free(dir_info);
         return ret;
     }
@@ -220,13 +239,13 @@ int create_file_src(Ecma119Image *img, IsoFile *iso, IsoFileSrc **src)
     off_t size;
 
     size = iso_stream_get_size(iso->stream);
-    if (size > (off_t)MAX_ISO_FILE_SECTION_SIZE && img->iso_level != 3) {
+    if (size > (off_t)MAX_ISO_FILE_SECTION_SIZE && img->opts->iso_level != 3) {
         char *ipath = iso_tree_get_node_path(ISO_NODE(iso));
-        ret = iso_msg_submit(img->image->id, ISO_FILE_TOO_BIG, 0,
-                              "File \"%s\" can't be added to image because "
-                              "is greater than 4GB", ipath);
+        iso_msg_submit(img->image->id, ISO_FILE_TOO_BIG, 0,
+                              "File \"%s\" cannot be added to image because "
+                              "its size is 4 GiB or larger", ipath);
         free(ipath);
-        return ret;
+        return ISO_FILE_TOO_BIG;
     }
     ret = iso_file_src_create(img, iso, src);
     if (ret < 0) {
@@ -337,7 +356,8 @@ void ecma119_node_free(Ecma119Node *node)
         for (i = 0; i < node->info.dir->nchildren; i++) {
             ecma119_node_free(node->info.dir->children[i]);
         }
-        free(node->info.dir->children);
+        if (node->info.dir->children != NULL)
+            free(node->info.dir->children);
         free(node->info.dir);
     }
     free(node->iso_name);
@@ -345,6 +365,35 @@ void ecma119_node_free(Ecma119Node *node)
     free(node);
 }
 
+
+static
+int add_to_hidden_list(Ecma119Image *image, IsoFileSrc *src)
+{
+    int ret;
+    struct iso_filesrc_list_item *item;
+
+    LIBISO_ALLOC_MEM(item, struct iso_filesrc_list_item, 1);
+    item->src = src;
+    item->next = image->ecma119_hidden_list;
+    image->ecma119_hidden_list = item;
+    ret = ISO_SUCCESS;
+ex:
+    return ret;
+}
+
+
+int iso_filesrc_list_destroy(struct iso_filesrc_list_item **start_item)
+{
+    struct iso_filesrc_list_item *item, *next;
+
+    for (item = *start_item; item != NULL; item = next) {
+        next = item->next;
+        LIBISO_FREE_MEM(item);
+    }
+    return ISO_SUCCESS;
+}
+ 
+
 /**
  * @param flag
  *      bit0= iso is in a hidden directory. Thus hide it.
@@ -361,10 +410,12 @@ int create_tree(Ecma119Image *image, IsoNode *iso, Ecma119Node **tree,
     int max_path;
     char *iso_name= NULL, *ipath = NULL;
     IsoFileSrc *src = NULL;
+    IsoWriteOpts *opts;
 
     if (image == NULL || iso == NULL || tree == NULL) {
         return ISO_NULL_POINTER;
     }
+    opts = image->opts;
     *tree = NULL;
 
     hidden = flag & 1;
@@ -385,16 +436,16 @@ int create_tree(Ecma119Image *image, IsoNode *iso, Ecma119Node **tree,
             goto ex;
         }
         max_path = pathlen + 1 + (iso_name ? strlen(iso_name) : 0);
-        if (!image->rockridge) {
+        if (!opts->rockridge) {
             if ((iso->type == LIBISO_DIR && depth > 8) &&
-                !image->allow_deep_paths) {
-	        ipath = iso_tree_get_node_path(iso);
+                !opts->allow_deep_paths) {
+                ipath = iso_tree_get_node_path(iso);
                 ret = iso_msg_submit(image->image->id, ISO_FILE_IMGPATH_WRONG,
                                      0, "File \"%s\" can't be added, "
                                      "because directory depth "
                                      "is greater than 8.", ipath);
                 goto ex;
-            } else if (max_path > 255 && !image->allow_longer_paths) {
+            } else if (max_path > 255 && !opts->allow_longer_paths) {
                 ipath = iso_tree_get_node_path(iso);
                 ret = iso_msg_submit(image->image->id, ISO_FILE_IMGPATH_WRONG,
                                      0, "File \"%s\" can't be added, "
@@ -409,6 +460,9 @@ int create_tree(Ecma119Image *image, IsoNode *iso, Ecma119Node **tree,
     case LIBISO_FILE:
         if (hidden) {
             ret = create_file_src(image, (IsoFile *) iso, &src);
+            if (ret <= 0)
+                goto ex;
+            ret = add_to_hidden_list(image, src); 
         } else {
             ret = create_file(image, (IsoFile*)iso, &node);
         }
@@ -418,7 +472,7 @@ int create_tree(Ecma119Image *image, IsoNode *iso, Ecma119Node **tree,
             ret = 0; /* Hidden means non-existing */
             goto ex;
         }
-        if (image->rockridge) {
+        if (opts->rockridge) {
             ret = create_symlink(image, (IsoSymlink*)iso, &node);
         } else {
             /* symlinks are only supported when RR is enabled */
@@ -434,7 +488,7 @@ int create_tree(Ecma119Image *image, IsoNode *iso, Ecma119Node **tree,
             ret = 0; /* Hidden means non-existing */
             goto ex;
         }
-        if (image->rockridge) {
+        if (opts->rockridge) {
             ret = create_special(image, (IsoSpecial*)iso, &node);
         } else {
             /* special files are only supported when RR is enabled */
@@ -449,6 +503,9 @@ int create_tree(Ecma119Image *image, IsoNode *iso, Ecma119Node **tree,
         if (image->eltorito) {
             if (hidden) {
                 ret = el_torito_catalog_file_src_create(image, &src);
+                if (ret <= 0)
+                    goto ex;
+                ret = add_to_hidden_list(image, src); 
             } else {
                 ret = create_boot_cat(image, (IsoBoot*)iso, &node);
             }
@@ -472,9 +529,9 @@ int create_tree(Ecma119Image *image, IsoNode *iso, Ecma119Node **tree,
                     image->rr_reloc_node = node;
                 } else if (depth == 2) {
                     /* Directories in root may be used as relocation dir */
-                    if (image->rr_reloc_dir != NULL)
-                        if (image->rr_reloc_dir[0] != 0 &&
-                            strcmp(iso->name, image->rr_reloc_dir) == 0)
+                    if (opts->rr_reloc_dir != NULL)
+                        if (opts->rr_reloc_dir[0] != 0 &&
+                            strcmp(iso->name, opts->rr_reloc_dir) == 0)
                             image->rr_reloc_node = node;
                 }
             }
@@ -540,13 +597,15 @@ int cmp_node_name(const void *f1, const void *f2)
 
 /**
  * Sorts a the children of each directory in the ECMA-119 tree represented
- * by \p root, acording to the order specified in ECMA-119, section 9.3.
+ * by \p root, according to the order specified in ECMA-119, section 9.3.
  */
 static
 void sort_tree(Ecma119Node *root)
 {
     size_t i;
 
+    if (root->info.dir->children == NULL)
+        return;
     qsort(root->info.dir->children, root->info.dir->nchildren, sizeof(void*),
           cmp_node_name);
     for (i = 0; i < root->info.dir->nchildren; i++) {
@@ -575,6 +634,9 @@ int mangle_single_dir(Ecma119Image *img, Ecma119Node *dir, int max_file_len,
     nchildren = dir->info.dir->nchildren;
     children = dir->info.dir->children;
 
+    if (nchildren <= 0)
+        return ISO_SUCCESS; /* nothing to do */
+
     /* a hash table will temporary hold the names, for fast searching */
     ret = iso_htable_create((nchildren * 100) / 80, iso_str_hash,
                             (compare_function_t)strcmp, &table);
@@ -592,6 +654,7 @@ int mangle_single_dir(Ecma119Image *img, Ecma119Node *dir, int max_file_len,
     for (i = 0; i < nchildren; ++i) {
         char *name, *ext;
         char full_name[40];
+        const int full_max_len = 40 - 1;
         int max; /* computed max len for name, without extension */
         int j = i;
         int digits = 1; /* characters to change per name */
@@ -606,7 +669,7 @@ int mangle_single_dir(Ecma119Image *img, Ecma119Node *dir, int max_file_len,
             continue;
         }
 
-        if (img->untranslated_name_len) {
+        if (img->opts->untranslated_name_len) {
             /* This should not happen because no two IsoNode names should be
                identical and only unaltered IsoNode names should be seen here.
                Thus the Ema119Node names should be unique.
@@ -630,12 +693,14 @@ int mangle_single_dir(Ecma119Image *img, Ecma119Node *dir, int max_file_len,
             int change = 0; /* number to be written */
 
             /* copy name to buffer */
-            strcpy(full_name, children[i]->iso_name);
+            strncpy(full_name, children[i]->iso_name, full_max_len);
+            full_name[full_max_len] = 0;
 
             /* compute name and extension */
             dot = strrchr(full_name, '.');
             if (dot != NULL &&
-                (children[i]->type != ECMA119_DIR || img->allow_dir_id_ext)) {
+                (children[i]->type != ECMA119_DIR ||
+                 img->opts->allow_dir_id_ext)) {
 
                 /*
                  * File (normally not dir) with extension
@@ -801,11 +866,11 @@ int mangle_tree(Ecma119Image *img, Ecma119Node *dir, int recurse)
     int max_file, max_dir;
     Ecma119Node *root;
 
-    if (img->untranslated_name_len > 0) {
-        max_file = max_dir = img->untranslated_name_len;
-    } else if (img->max_37_char_filenames) {
+    if (img->opts->untranslated_name_len > 0) {
+        max_file = max_dir = img->opts->untranslated_name_len;
+    } else if (img->opts->max_37_char_filenames) {
         max_file = max_dir = 37;
-    } else if (img->iso_level == 1) {
+    } else if (img->opts->iso_level == 1) {
         max_file = 12; /* 8 + 3 + 1 */
         max_dir = 8;
     } else {
@@ -973,9 +1038,9 @@ int reorder_tree(Ecma119Image *img, Ecma119Node *dir,
         /* dir is now the relocated Ecma119Node */
         pathlen = 37 + 1; /* The dir name might get longer by mangling */
         level = 2;
-        if (img->rr_reloc_dir != NULL) {
+        if (img->opts->rr_reloc_dir != NULL) {
             pathlen += strlen(img->rr_reloc_node->iso_name) + 1;
-            if(img->rr_reloc_dir[0] != 0)
+            if(img->opts->rr_reloc_dir[0] != 0)
               level = 3;
         }
     }
@@ -1095,6 +1160,11 @@ int family_set_ino(Ecma119Image *img, Ecma119Node **nodes, size_t family_start,
         */
         if (img_ino == prev_ino)
             img_ino = 0;
+
+	/* Accept only if it is within the 32 bit range. */
+        if (((uint64_t) img_ino) > 0xffffffff)
+            img_ino = 0;
+
     }
     if (img_ino == 0) {
         img_ino = img_give_ino_number(img->image, 0);
@@ -1128,7 +1198,7 @@ int match_hardlinks(Ecma119Image *img, Ecma119Node *dir, int flag)
         goto ex;
 
     /* Sort according to id tuples, IsoFileSrc identity, properties, xattr. */
-    if (img->hardlinks)
+    if (img->opts->hardlinks)
         qsort(nodes, node_count, sizeof(Ecma119Node *), ecma119_node_cmp_hard);
     else
         qsort(nodes, node_count, sizeof(Ecma119Node *),
@@ -1198,9 +1268,9 @@ int ecma119_tree_create(Ecma119Image *img)
         return ret;
     }
 
-    if (img->rockridge && !img->allow_deep_paths) {
+    if (img->opts->rockridge && !img->opts->allow_deep_paths) {
 
-        /* Relocate deep directories, acording to RRIP, 4.1.5 */
+        /* Relocate deep directories, according to RRIP, 4.1.5 */
         ret = reorder_tree(img, root, 1, 0);
         if (ret < 0) {
             return ret;
