@@ -30,6 +30,11 @@
 #include <libgen.h>
 #include <string.h>
 
+/* O_BINARY is needed for Cygwin but undefined elsewhere */
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
+
 static
 int iso_file_source_new_lfs(IsoFileSource *parent, const char *name, 
                             IsoFileSource **src);
@@ -222,7 +227,7 @@ int lfs_open(IsoFileSource *src)
         data->info.dir = opendir(path);
         data->openned = data->info.dir ? 2 : 0;
     } else {
-        data->info.fd = open(path, O_RDONLY);
+        data->info.fd = open(path, O_RDONLY | O_BINARY);
         data->openned = data->info.fd != -1 ? 1 : 0;
     }
     free(path);
@@ -282,6 +287,9 @@ static
 int lfs_read(IsoFileSource *src, void *buf, size_t count)
 {
     _LocalFsFileSource *data;
+    size_t to_read, done = 0;
+    int ret;
+    uint8_t *buf8;
 
     if (src == NULL || buf == NULL) {
         return ISO_NULL_POINTER;
@@ -293,28 +301,28 @@ int lfs_read(IsoFileSource *src, void *buf, size_t count)
     data = src->data;
     switch (data->openned) {
     case 1: /* not dir */
-        {
-            int ret;
-            ret = read(data->info.fd, buf, count);
+        buf8 = (uint8_t *) buf; /* for pointer arithmetic */
+        for (to_read = count; to_read > 0; to_read = count - done) {
+            if (to_read > 1024 * 1024)
+                to_read = 1024 * 1024;
+            ret = read(data->info.fd, buf8 + done, to_read);
             if (ret < 0) {
                 /* error on read */
                 switch (errno) {
                 case EINTR:
-                    ret = ISO_INTERRUPTED;
-                    break;
+                    return ISO_INTERRUPTED;
                 case EFAULT:
-                    ret = ISO_OUT_OF_MEM;
-                    break;
+                    return ISO_OUT_OF_MEM;
                 case EIO:
-                    ret = ISO_FILE_READ_ERROR;
-                    break;
-                default:
-                    ret = ISO_FILE_ERROR;
-                    break;
+                    return ISO_FILE_READ_ERROR;
                 }
+                return ISO_FILE_ERROR;
             }
-            return ret;
+            if (ret == 0) /* EOF */
+        break;
+            done += ret;
         }
+        return done;
     case 2: /* directory */
         return ISO_FILE_IS_DIR;
     default:
@@ -492,7 +500,8 @@ static
 int lfs_get_aa_string(IsoFileSource *src, unsigned char **aa_string, int flag)
 {
     int ret;
-    size_t num_attrs = 0, *value_lengths = NULL, result_len, sret;
+    size_t num_attrs = 0, *value_lengths = NULL, result_len;
+    ssize_t sret;
     char *path = NULL, **names = NULL, **values = NULL;
     unsigned char *result = NULL;
 
@@ -525,10 +534,10 @@ int lfs_get_aa_string(IsoFileSource *src, unsigned char **aa_string, int flag)
     else {
         sret = aaip_encode(num_attrs, names,
                            value_lengths, values, &result_len, &result, 0);
-        if (sret == 0) {
-            ret = ISO_OUT_OF_MEM;
+        if (sret < 0) {
+            ret = sret;
             goto ex;
-         }
+        }
     }
     *aa_string = result;
     ret = 1;
@@ -536,7 +545,7 @@ ex:;
     if (path != NULL)
         free(path);
     if (names != NULL || value_lengths != NULL || values != NULL)
-        aaip_get_attr_list(path, &num_attrs, &names, &value_lengths, &values,
+        aaip_get_attr_list(NULL, &num_attrs, &names, &value_lengths, &values,
                            1 << 15); /* free memory */
     return ret;
 }
