@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2007 Vreixo Formoso
- * Copyright (c) 2009 - 2013 Thomas Schmitt
+ * Copyright (c) 2009 - 2015 Thomas Schmitt
  *
  * This file is part of the libisofs project; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License version 2 
@@ -22,6 +22,100 @@
 #include <string.h>
 #include <stdio.h>
 
+
+int iso_imported_sa_new(struct iso_imported_sys_area **boots, int flag)
+{
+    struct iso_imported_sys_area *b;
+
+    *boots = NULL;
+    b = calloc(1, sizeof(struct iso_imported_sys_area));
+    if (b == NULL)
+        return ISO_OUT_OF_MEM;
+
+    b->mbr_req = NULL;
+    b->apm_req = NULL;
+
+    b->gpt_req = NULL;
+    b->gpt_backup_comments = NULL;
+
+    b->mips_boot_file_paths = NULL;
+    b->mips_vd_entries = NULL;
+
+    b->sparc_disc_label = NULL;
+    b->sparc_core_node = NULL;
+    b->sparc_entries = NULL;
+
+    b->hppa_cmdline = NULL;
+    b->hppa_bootloader = NULL;
+    b->hppa_kernel_32 = NULL;
+    b->hppa_kernel_64 = NULL;
+    b->hppa_ramdisk = NULL;
+
+    b->alpha_boot_image = NULL;
+
+    *boots = b;
+    return 1;
+}
+
+int iso_imported_sa_unref(struct iso_imported_sys_area **boots, int flag)
+{
+    int i;
+    struct iso_imported_sys_area *b;
+
+    b = *boots;
+    if (b == NULL)
+        return 2;
+    if (b->refcount > 0)
+        b->refcount--;
+    if (b->refcount > 0)
+        return 2;
+
+    if (b->mbr_req != NULL) {
+        for (i = 0; i < b->mbr_req_count; i++)
+            LIBISO_FREE_MEM(b->mbr_req[i]);
+        LIBISO_FREE_MEM(b->mbr_req);
+    }
+    if (b->apm_req != NULL) {
+        for (i = 0; i < b->apm_req_count; i++)
+            LIBISO_FREE_MEM(b->apm_req[i]);
+        LIBISO_FREE_MEM(b->apm_req);
+    }
+    if (b->gpt_req != NULL) {
+        for (i = 0; i < b->gpt_req_count; i++)
+            LIBISO_FREE_MEM(b->gpt_req[i]);
+        LIBISO_FREE_MEM(b->gpt_req);
+    }
+    LIBISO_FREE_MEM(b->gpt_backup_comments);
+
+    if (b->mips_boot_file_paths != NULL) {
+        for (i = 0; i < b->num_mips_boot_files; i++)
+            LIBISO_FREE_MEM(b->mips_boot_file_paths[i]);
+        LIBISO_FREE_MEM(b->mips_boot_file_paths);
+    }
+    if (b->mips_vd_entries != NULL) {
+        for (i = 0; i < b->num_mips_boot_files; i++)
+            LIBISO_FREE_MEM(b->mips_vd_entries[i]);
+        LIBISO_FREE_MEM(b->mips_vd_entries);
+    }
+    LIBISO_FREE_MEM(b->mipsel_boot_file_path);
+
+    LIBISO_FREE_MEM(b->sparc_disc_label);
+    if (b->sparc_core_node != NULL)
+        iso_node_unref((IsoNode *) b->sparc_core_node);
+    LIBISO_FREE_MEM(b->sparc_entries);
+
+    LIBISO_FREE_MEM(b->hppa_cmdline);
+    LIBISO_FREE_MEM(b->hppa_bootloader);
+    LIBISO_FREE_MEM(b->hppa_kernel_32);
+    LIBISO_FREE_MEM(b->hppa_kernel_64);
+    LIBISO_FREE_MEM(b->hppa_ramdisk);
+    LIBISO_FREE_MEM(b->alpha_boot_image);
+    LIBISO_FREE_MEM(b);
+    *boots = NULL;
+    return 1;
+}
+
+
 /**
  * Create a new image, empty.
  *
@@ -78,14 +172,25 @@ int iso_image_new(const char *name, IsoImage **image)
         img->volset_id = strdup(name);
         img->volume_id = strdup(name);
     }
+    memset(img->application_use, 0, 512);
     img->system_area_data = NULL;
     img->system_area_options = 0;
     img->num_mips_boot_files = 0;
     for (i = 0; i < 15; i++)
          img->mips_boot_file_paths[i] = NULL;
     img->sparc_core_node = NULL;
+    img->hppa_cmdline= NULL;
+    img->hppa_bootloader = NULL;
+    img->hppa_kernel_32 = NULL;
+    img->hppa_kernel_64 = NULL;
+    img->hppa_ramdisk = NULL;
+    img->alpha_boot_image = NULL;
+    img->import_src = NULL;
     img->builder_ignore_acl = 1;
     img->builder_ignore_ea = 1;
+    img->truncate_mode = 1;
+    img->truncate_length = LIBISOFS_NODE_NAME_MAX;
+    img->truncate_buffer[0] = 0;
     img->inode_counter = 0;
     img->used_inodes = NULL;
     img->used_inodes_start = 0;
@@ -96,6 +201,8 @@ int iso_image_new(const char *name, IsoImage **image)
     img->generator_is_running = 0;
     for (i = 0; i < ISO_HFSPLUS_BLESS_MAX; i++)
         img->hfsplus_blessed[i] = NULL;
+    img->collision_warnings = 0;
+    img->imported_sa_info = NULL;
 
     *image = img;
     return ISO_SUCCESS;
@@ -139,6 +246,11 @@ void iso_image_unref(IsoImage *image)
         iso_image_give_up_mips_boot(image, 0);
         if (image->sparc_core_node != NULL)
             iso_node_unref((IsoNode *) image->sparc_core_node);
+        iso_image_set_hppa_palo(image, NULL, NULL, NULL, NULL, NULL, 1);
+        if (image->alpha_boot_image != NULL)
+            free(image->alpha_boot_image);
+        if (image->import_src != NULL)
+            iso_data_source_unref(image->import_src);
         free(image->volset_id);
         free(image->volume_id);
         free(image->publisher_id);
@@ -157,6 +269,7 @@ void iso_image_unref(IsoImage *image)
         if (image->system_area_data != NULL)
             free(image->system_area_data);
         iso_image_free_checksums(image, 0);
+        iso_imported_sa_unref(&(image->imported_sa_info), 0);
         free(image);
     }
 }
@@ -349,13 +462,18 @@ int iso_image_set_pvd_times(IsoImage *image,
     if (creation_time == NULL || modification_time == NULL ||
         expiration_time == NULL || effective_time == NULL)
         return ISO_NULL_POINTER;
-    image->creation_time = strdup(creation_time);
-    image->modification_time = strdup(modification_time);
-    image->expiration_time = strdup(expiration_time);
-    image->effective_time = strdup(effective_time);
+    image->creation_time = calloc(18, 1);   /* Surely including a trailing 0 */
+    image->modification_time = calloc(18, 1);
+    image->expiration_time = calloc(18, 1);
+    image->effective_time = calloc(18, 1);
     if (image->creation_time == NULL || image->modification_time == NULL ||
         image->expiration_time == NULL || image->effective_time == NULL)
         return ISO_OUT_OF_MEM;
+    /* (If the string is too short, a non-zero timezone will not be stored) */
+    strncpy(image->creation_time, creation_time, 17);
+    strncpy(image->modification_time, modification_time, 17);
+    strncpy(image->expiration_time, expiration_time, 17);
+    strncpy(image->effective_time, effective_time, 17);
     return ISO_SUCCESS;
 }
 
@@ -373,6 +491,19 @@ int iso_image_get_pvd_times(IsoImage *image,
     return ISO_SUCCESS;
 }
 
+void iso_image_set_app_use(IsoImage *image, const char *app_use_data,
+                           int count)
+{
+    if (count < 0)
+        count= 0;
+    else if(count > 512)
+        count= 512;
+    if (count > 0)
+        memcpy(image->application_use, app_use_data, count);
+    if (count < 512)
+        memset(image->application_use + count, 0, 512 - count);
+}
+
 int iso_image_get_msg_id(IsoImage *image)
 {
     return image->id;
@@ -392,21 +523,74 @@ static
 int dir_update_size(IsoImage *image, IsoDir *dir)
 {
     IsoNode *pos;
+    int ret;
+
+#ifdef Libisofs_update_sizes_abortablE
+    char *path= NULL;
+    IsoStream *base_stream;
+    int cancel_ret, ret;
+    uint32_t lba;
+#endif
+
     pos = dir->children;
     while (pos) {
-        int ret = 1;
         if (pos->type == LIBISO_FILE) {
             ret = iso_stream_update_size(ISO_FILE(pos)->stream);
         } else if (pos->type == LIBISO_DIR) {
             /* recurse */
             ret = dir_update_size(image, ISO_DIR(pos));
+
+#ifdef Libisofs_update_sizes_abortablE
+            if (ret == ISO_CANCELED)
+                return ret; /* Message already issued by dir_update_size */
+#endif
+
+        } else {
+            ret = 1;
         }
+
+#ifdef Libisofs_update_sizes_abortablE
+
+        /* This would report error and abort according to severity threshold.
+           But it is desirable to let the update_size crawler continue
+           its work after e.g. a file has vanished from hard disk.
+           So normally this macro case should be disabled.
+        */
+
         if (ret < 0) {
-            ret = iso_msg_submit(image->id, ret, 0, NULL);
-            if (ret < 0) {
-                return ret; /* cancel due error threshold */
+            cancel_ret = iso_msg_submit(image->id, ret, 0, NULL);
+            path = iso_tree_get_node_path(pos);
+            if (path != NULL) {
+                iso_msg_submit(image->id, ret, 0,
+                               "ISO path  : %s", path);
+                free(path);
             }
+            /* Report source path with streams which do not come from
+               the loaded ISO filesystem */
+            if (pos->type == LIBISO_FILE &&
+                iso_node_get_old_image_lba(pos, &lba, 0) == 0) {
+                base_stream = iso_stream_get_input_stream(
+                                                     ISO_FILE(pos)->stream, 1);
+                if (base_stream == NULL)
+                    base_stream = ISO_FILE(pos)->stream;
+                path = iso_stream_get_source_path(base_stream, 0);
+                if (path != NULL) {
+                    iso_msg_submit(image->id, ret, 0,
+                                   "Local path: %s", path);
+                    free(path);
+                }
+            }
+            if (cancel_ret < 0)
+                return cancel_ret; /* cancel due error threshold */
         }
+
+#else
+
+        if (ret < 0)
+            ret = 1; /* ignore error */
+
+#endif /* ! Libisofs_update_sizes_abortablE */
+
         pos = pos->next;
     }
     return ISO_SUCCESS;
@@ -502,7 +686,8 @@ ex:;
 
 
 /**
- * A global counter for inode numbers for the ISO image filesystem.
+ * A global counter for Rock Ridge inode numbers in the ISO image filesystem.
+ *
  * On image import it gets maxed by the eventual inode numbers from PX
  * entries. Up to the first 32 bit rollover it simply increments the counter.
  * After the first rollover it uses a look ahead bitmap which gets filled
@@ -512,13 +697,13 @@ ex:;
  * @param image The image where the number shall be used
  * @param flag  bit0= reset count (Caution: image must get new inos then)
  * @return
- *     Since ino_t 0 is used as default and considered self-unique,
+ *     Since 0 is used as default and considered self-unique,
  *     the value 0 should only be returned in case of error.
  */
-ino_t img_give_ino_number(IsoImage *image, int flag)
+uint32_t img_give_ino_number(IsoImage *image, int flag)
 {
     int ret;
-    ino_t new_ino, ino_idx;
+    uint64_t new_ino, ino_idx;
     static uint64_t limit = 0xffffffff;
 
     if (flag & 1) {
@@ -528,10 +713,10 @@ ino_t img_give_ino_number(IsoImage *image, int flag)
         image->used_inodes = NULL;
         image->used_inodes_start = 0;
     }
-    new_ino = image->inode_counter + 1;
+    new_ino = ((uint64_t) image->inode_counter) + 1;
     if (image->used_inodes == NULL) {
         if (new_ino > 0 && new_ino <= limit) {
-            image->inode_counter = new_ino;
+            image->inode_counter = (uint32_t) new_ino;
             return image->inode_counter;
         }
     }
@@ -791,3 +976,155 @@ int iso_image_get_sparc_core(IsoImage *img, IsoFile **sparc_core, int flag)
 }
 
 
+/* @param flag
+           bit0= Let NULL parameters free the corresponding image properties.
+                 Else only the non-NULL parameters of this call have an effect.
+*/
+static int hppa_palo_set_path(IsoImage *img, char *path, char **target,
+                              char *what, int flag)
+{
+    int ret, err;
+    IsoNode *node;
+    IsoFile *file;
+
+    if (path == NULL && !(flag & 1))
+        return ISO_SUCCESS;
+    if (iso_clone_mgtd_mem(path, target, 0) < 0)
+        return ISO_OUT_OF_MEM;
+    if (path == NULL)
+        return ISO_SUCCESS;
+    ret = iso_tree_path_to_node(img, path, &node);
+    if (ret < 0)
+        return ret;
+    if (ret == 0) {
+        iso_msg_submit(img->id, ISO_BOOT_FILE_MISSING, 0,
+                       "Cannot find in ISO image: %s file '%s'", what, path);
+        return ISO_BOOT_FILE_MISSING;
+    }
+    if (iso_node_get_type(node) != LIBISO_FILE) {
+        err = ISO_HPPA_PALO_NOTREG;
+        if (strncmp(what, "DEC Alpha", 9) == 0)
+            err = ISO_ALPHA_BOOT_NOTREG;
+        iso_msg_submit(img->id, err, 0,
+                       "%s file is not a data file: '%s'", what, path);
+        return err;
+    }
+    file = (IsoFile *) node;
+    if (!(file->explicit_weight || file->from_old_session))
+        file->sort_weight = 2;
+    return ISO_SUCCESS;
+}
+
+
+/* API */
+/* @param flag
+          Bitfield for control purposes
+           bit0= Let NULL parameters free the corresponding image properties.
+                 Else only the non-NULL parameters of this call have an effect.
+*/
+int iso_image_set_hppa_palo(IsoImage *img, char *cmdline, char *bootloader,
+                            char *kernel_32, char *kernel_64, char *ramdisk,
+                            int flag)
+{
+    int ret;
+    static char *what = "HP-PA PALO";
+
+    if (cmdline != NULL || (flag & 1))
+        if (iso_clone_mgtd_mem(cmdline, &(img->hppa_cmdline), 0) < 0)
+            return ISO_OUT_OF_MEM;
+    ret = hppa_palo_set_path(img, bootloader, &(img->hppa_bootloader), what,
+                             flag & 1);
+    if (ret < 0)
+        return ret;
+    ret = hppa_palo_set_path(img, kernel_32, &(img->hppa_kernel_32), what,
+                             flag & 1);
+    if (ret < 0)
+        return ret;
+    ret = hppa_palo_set_path(img, kernel_64, &(img->hppa_kernel_64), what,
+                             flag & 1);
+    if (ret < 0)
+        return ret;
+    ret = hppa_palo_set_path(img, ramdisk, &(img->hppa_ramdisk), what,
+                             flag & 1);
+    if (ret < 0)
+        return ret;
+    return ISO_SUCCESS;
+}
+
+
+/* API */
+int iso_image_get_hppa_palo(IsoImage *img, char **cmdline, char **bootloader,
+                            char **kernel_32, char **kernel_64, char **ramdisk)
+{
+    *cmdline = img->hppa_cmdline;
+    *bootloader = img->hppa_bootloader;
+    *kernel_32 = img->hppa_kernel_32;
+    *kernel_64 = img->hppa_kernel_64;
+    *ramdisk  = img->hppa_ramdisk;
+    return ISO_SUCCESS;
+}
+
+
+/* API */
+int iso_image_set_alpha_boot(IsoImage *img, char *boot_loader_path, int flag)
+{
+    int ret;
+
+    ret = hppa_palo_set_path(img, boot_loader_path, &(img->alpha_boot_image),
+                             "DEC Alpha Bootloader", 1);
+    if (ret < 0)
+        return ret;
+    return ISO_SUCCESS;
+}
+
+
+/* API */
+int iso_image_get_alpha_boot(IsoImage *img, char **boot_loader_path)
+{
+    *boot_loader_path = img->alpha_boot_image;
+    return ISO_SUCCESS;
+}
+
+
+/* API */
+int iso_image_set_truncate_mode(IsoImage *img, int mode, int length)
+{
+    if (mode < 0 || mode > 1)
+        return ISO_WRONG_ARG_VALUE;
+    if (length < 64 || length > LIBISOFS_NODE_NAME_MAX)
+        return ISO_WRONG_ARG_VALUE;
+    img->truncate_mode = mode;
+    img->truncate_length = length;
+    return ISO_SUCCESS;
+}
+
+/* API */
+int iso_image_get_truncate_mode(IsoImage *img, int *mode, int *length)
+{
+    *mode = img->truncate_mode;
+    *length = img->truncate_length;
+    return ISO_SUCCESS;
+}
+
+/* Warning: Not thread-safe */
+int iso_image_truncate_name(IsoImage *image, const char *name, char **namept,
+                            int flag)
+{
+    int ret;
+
+    if (name == NULL) 
+        return ISO_NULL_POINTER;
+
+    if ((int) strlen(name) <= image->truncate_length) {
+        *namept = (char *) name;
+        return ISO_SUCCESS;
+    }
+    *namept = image->truncate_buffer;
+    if (name != image->truncate_buffer)
+        strncpy(image->truncate_buffer, name, 4095);
+    image->truncate_buffer[4095] = 0;
+    ret = iso_truncate_rr_name(image->truncate_mode, image->truncate_length,
+                               image->truncate_buffer, 0);
+    return ret;
+}
+
