@@ -547,6 +547,12 @@ const char *iso_error_to_msg(int errcode)
         return "Cannot derive GPT GUID from undefined pseudo-UUID volume timestamp";
     case ISO_BAD_GPT_GUID_MODE:
         return "Unrecognized GPT disk GUID setup mode";
+    case ISO_NO_ROOT_DIR:
+        return "Unable to obtain root directory";
+    case ISO_SUSP_WRONG_CE_SIZE:
+        return "Zero sized, oversized, or mislocated SUSP CE area found";
+    case ISO_MULTI_OVER_IMPORTED:
+        return "Multi-session would overwrite imported_iso interval";
     default:
         return "Unknown error";
     }
