@@ -1,11 +1,12 @@
 /*
  * Copyright (c) 2007 Vreixo Formoso
  * Copyright (c) 2007 Mario Danic
- * Copyright (c) 2009 Thomas Schmitt
+ * Copyright (c) 2009 - 2015 Thomas Schmitt
  * 
  * This file is part of the libisofs project; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License version 2 as 
- * published by the Free Software Foundation. See COPYING file for details.
+ * modify it under the terms of the GNU General Public License version 2 
+ * or later as published by the Free Software Foundation. 
+ * See COPYING file for details.
  */
 
 /**
@@ -62,10 +63,22 @@ struct susp_info
     uint32_t ce_block;
     uint32_t ce_len;
 
+    /* Storage for Continuation Area for a whole directory */
     size_t n_ce_susp_fields;
     uint8_t **ce_susp_fields;
+
+    /* The number of allocated members in ce_susp_fields */
+    size_t alloc_ce_susp_fields;
+
+    /* Marks the start index in ce_susp_fields of the current node */
+    size_t current_ce_start;
+
 };
 
+/* Step to increase allocated size of susp_info.ce_susp_fields */
+#define ISO_SUSP_CE_ALLOC_STEP 16
+
+
 /* SUSP 5.1 */
 struct susp_CE {
     uint8_t block[8];
@@ -185,11 +198,13 @@ struct susp_sys_user_entry
  *      Available space in the System Use Area for the directory record.
  * @param ce
  *      Will be filled with the space needed in a CE
+ * @param base_ce
+ *      Fill of continuation area by previous nodes of same dir
  * @return
  *      The size needed for the RR entries in the System Use Area
  */
 size_t rrip_calc_len(Ecma119Image *t, Ecma119Node *n, int type, size_t space,
-                     size_t *ce);
+                     size_t *ce, size_t base_ce);
 
 /**
  * Fill a struct susp_info with the RR/SUSP entries needed for a given
@@ -239,7 +254,7 @@ typedef struct susp_iterator SuspIterator;
 
 SuspIterator *
 susp_iter_new(IsoDataSource *src, struct ecma119_dir_record *record, 
-              uint8_t len_skp, int msgid);
+              uint32_t fs_blocks, uint8_t len_skp, int msgid);
 
 /**
  * Get the next SUSP System User Entry using given iterator.
@@ -251,7 +266,8 @@ susp_iter_new(IsoDataSource *src, struct ecma119_dir_record *record,
  * @return
  *      1 on success, 0 if no more entries, < 0 error
  */
-int susp_iter_next(SuspIterator *iter, struct susp_sys_user_entry **sue);
+int susp_iter_next(SuspIterator *iter, struct susp_sys_user_entry **sue,
+                   int flag);
 
 /**
  * Free a given susp iterator.
@@ -345,4 +361,12 @@ int read_zisofs_ZF(struct susp_sys_user_entry *zf, uint8_t algorithm[2],
                    uint8_t *header_size_div4, uint8_t *block_size_log2,
                    uint32_t *uncompressed_size, int flag);
 
+/**
+ * Convert a RR filename to the requested charset.
+ * @param flag   bit0= do not issue error messages
+ */
+int iso_get_rr_name(IsoWriteOpts *opts, char *input_charset,
+                    char *output_charset, int imgid,
+                    char *str, char **name, int flag);
+
 #endif /* LIBISO_ROCKRIDGE_H */
